package com.example.demo;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo.ageRatings.model.AgeRatingEntity;
import com.example.demo.ageRatings.service.AgeRatingService;
import com.example.demo.core.error.NotFoundException;
import com.example.demo.movies.model.MovieEntity;
import com.example.demo.movies.service.MovieService;
import com.example.demo.types.model.TypeEntity;
import com.example.demo.types.service.TypeService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class MovieServiceTests {
	@Autowired
	private TypeService typeService;

	@Autowired
	private AgeRatingService ageRatingService;

	@Autowired
	private MovieService movieService;

	@Test
	void getTest() {
		Assertions.assertThrows(NotFoundException.class, () -> movieService.get(0L));
	}

	@Test
	@Order(1)
	void createTest() {
		movieService.deleteAll();

		final var type1 = typeService.create(new TypeEntity(null, "Фильм"));
		final var type2 = typeService.create(new TypeEntity(null, "Сериал"));
		final var ageRating4 = ageRatingService.create(new AgeRatingEntity(null, "16+"));
		final var ageRating5 = ageRatingService.create(new AgeRatingEntity(null, "18+"));

		movieService.create(new MovieEntity(null, "Бойцовский клуб", type1, false,
				"src/assets/main-page/posters/PeakyBlinders.png",
				"Сотрудник страховой компании страдает хронической бессонницей и отчаянно пытается вырваться из мучительно скучной жизни...",
				1999, "США", "Интриги. Хаос. Мыло", "Дэвид Финчер", ageRating5,
				"https://www.youtube.com/embed/dfeUzm6KF4g?si=U2D-2WNsJzkcxlBX"));
		movieService.create(new MovieEntity(null, "Американский психопат", type1, true,
				"src/assets/main-page/posters/AmericanPsycho.jpg",
				"Днем он ничем не отличается от окружающих, и в толпе вы не обратите на него внимания...",
				2000, "США, Канада", "Респектабельная внешность дьявола", "Мэри Хэррон", ageRating5,
				"https://www.youtube.com/embed/PpUWjff_OcM?si=e7tSm4V6qYkz406B"));
		final MovieEntity last = movieService.create(new MovieEntity(null, "Острые козырьки", type2, false,
				"src/assets/main-page/posters/PeakyBlinders.png",
				"Сериал рассказывает историю восхождения одной ирландской семьи...",
				2013, "Великобритания", "The streets are theirs", "Энтони Бирн, Колм МакКарти", ageRating4,
				"https://www.youtube.com/embed/7cOfn_sjlXM?si=fxNoZjlUon1SZd70"));

		Assertions.assertEquals(3, movieService.getAll(0L).size());
		Assertions.assertEquals(2, movieService.getAll(1L).size());
		Assertions.assertEquals(last, movieService.get(3L));
	}

	@Test
	@Order(2)
	void updateTest() {
		final var type2 = typeService.create(new TypeEntity(null, "Сериал"));
		final var ageRating4 = ageRatingService.create(new AgeRatingEntity(null, "16+"));

		final String test = "TEST";
		final MovieEntity entity = movieService.get(3L);
		final String oldName = entity.getTitle();
		final MovieEntity newEntity = movieService.update(3L, new MovieEntity(null,
				test, type2, false,
				"src/assets/main-page/posters/PeakyBlinders.png",
				"Сериал рассказывает историю восхождения одной ирландской семьи...",
				2013, "Великобритания", "The streets are theirs", "Энтони Бирн, Колм МакКарти", ageRating4,
				"https://www.youtube.com/embed/7cOfn_sjlXM?si=fxNoZjlUon1SZd70"));

		Assertions.assertEquals(3, movieService.getAll(0L).size());
		Assertions.assertEquals(newEntity, movieService.get(3L));
		Assertions.assertEquals(test, newEntity.getTitle());
		Assertions.assertNotEquals(oldName, newEntity.getTitle());
	}

	@Test
	@Order(3)
	void deleteTest() {
		final var type2 = typeService.create(new TypeEntity(null, "Сериал"));
		final var ageRating4 = ageRatingService.create(new AgeRatingEntity(null, "16+"));

		movieService.delete(3L);
		Assertions.assertEquals(2, movieService.getAll(0L).size());
		final MovieEntity last = movieService.get(2L);
		Assertions.assertEquals(2L, last.getId());

		final MovieEntity newEntity = movieService.create(new MovieEntity(null,
				"Острые козырьки", type2, false,
				"src/assets/main-page/posters/PeakyBlinders.png",
				"Сериал рассказывает историю восхождения одной ирландской семьи...",
				2013, "Великобритания", "The streets are theirs", "Энтони Бирн, Колм МакКарти", ageRating4,
				"https://www.youtube.com/embed/7cOfn_sjlXM?si=fxNoZjlUon1SZd70"));
		Assertions.assertEquals(3, movieService.getAll(0L).size());
		Assertions.assertEquals(4L, newEntity.getId());
	}
}
